/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.common.data.manipulator.mutable.entity;

import org.spongepowered.api.data.key.Keys;
import org.spongepowered.api.data.manipulator.immutable.entity.ImmutableInvulnerabilityData;
import org.spongepowered.api.data.manipulator.mutable.entity.InvulnerabilityData;
import org.spongepowered.api.data.value.mutable.MutableBoundedValue;
import org.spongepowered.common.data.manipulator.immutable.entity.ImmutableSpongeInvulnerabilityData;
import org.spongepowered.common.data.manipulator.mutable.common.AbstractBoundedComparableData;
import org.spongepowered.common.data.util.ComparatorUtil;
import org.spongepowered.common.data.value.SpongeValueFactory;

public class SpongeInvulnerabilityData extends AbstractBoundedComparableData<Integer, InvulnerabilityData, ImmutableInvulnerabilityData>
        implements InvulnerabilityData {

    public SpongeInvulnerabilityData(int value, int minValue, int maxValue, int defaultValue) {
        super(InvulnerabilityData.class, value, Keys.INVULNERABILITY_TICKS, ComparatorUtil.intComparator(),
                ImmutableSpongeInvulnerabilityData.class, minValue, maxValue, defaultValue);
    }

    public SpongeInvulnerabilityData(int value) {
        this(value, 0, Integer.MAX_VALUE, 0);
    }

    public SpongeInvulnerabilityData() {
        this(0);
    }

    @Override
    public MutableBoundedValue<Integer> invulnerableTicks() {
        return SpongeValueFactory.boundedBuilder(Keys.INVULNERABILITY_TICKS)
                .actualValue(getValue())
                .defaultValue(0)
                .minimum(0)
                .maximum(Integer.MAX_VALUE)
                .build();
    }

    @Override
    protected MutableBoundedValue<Integer> getValueGetter() {
        return invulnerableTicks();
    }

    @Override
    public ImmutableInvulnerabilityData asImmutable() {
        return new ImmutableSpongeInvulnerabilityData(getValue());
    }

}
